﻿<%@ Page Language="C#" AutoEventWireup="true" %>

<%@ Import Namespace="JobTraQWebReference" %>
<%@ Import Namespace="System.Linq" %>
<%@ Import Namespace="System.Collections.Generic" %>

<script type="text/C#" runat="server">
	
	//this is the full url to the web service asmx file
	string WebServiceUrl = "http://localhost/jobtraq/api/JobTraQWebService.asmx";
	//These are the default login credentials. To log in manually, add "login=true" to the querystring. Leave blank to force login.
	string DefaultUserName = "";
	string DefaultPassword = "";
	
	JobTraQWebService ws; //holds a reference to the JobTraQ Web Service
	int EditTaskId; //the id of the task being edited or -1 if a new task is being created;
	int TaskTypeId; //the type of task to create or edit
	bool LoginMode = false;
	
	List<Control> JTControls = new List<Control>(); //holds a list of JT form controls for this task type
	Dictionary<string, string> TranslatedTaskValues; //holds the translated field values loaded from the database for the task being edited

	TaskField[] m_TypeFields;
	/// <summary>
	/// Loads and memoizes all JobTraQ task field info for this task type
	/// </summary>
	TaskField[] TypeFields
	{
		get { return m_TypeFields ?? (m_TypeFields = ws.GetAllAvailableTaskFields()); }
	}

	/// <summary>
	/// Creates a reference to the web server
	/// </summary>
	/// <param name="sender">The event sender</param>
	/// <param name="e">The EventArgs</param>
	protected void Page_Init(object sender, EventArgs e)
	{
		try
		{
			// Create a reference to the web service at the specified URL.
			ws = new JobTraQWebService
			{
				Url = WebServiceUrl,
				Timeout = 1000 * 60 // one minute, in milliseconds
			};
			
			//check for login mode
			LoginMode = "true".Equals(Request.QueryString["login"], StringComparison.OrdinalIgnoreCase);
			
			if (!LoginMode)
			{
				//if the user has logged in, use the Login stored in the Session, otherwises log in as the default
				JobTraQWebReference.Login JTLogin = Session["Login"] as JobTraQWebReference.Login;
				if (JTLogin == null)
				{
					try
					{
						Session["Login"] = JTLogin = ws.UserLogin(DefaultUserName, DefaultPassword);
					}
					catch (Exception)
					{
						//if the default login failed, force the user to log in manually
						NameValueCollection NewQuery = HttpUtility.ParseQueryString(Request.Url.Query);
						NewQuery["login"] = "true";
						Response.Redirect(Request.Path + "?" + NewQuery.ToString(), true);
					}
				}
				ws.UserAuthInformationValue = new UserAuthInformation { UserID = JTLogin.UserId, UserToken = JTLogin.UserToken };
			}
		}
		catch (Exception ex)
		{
			ShowError(ex);
		}
	}

	/// <summary>
	/// Handles any AJAX requests from the client
	/// </summary>
	/// <param name="sender">The event sender</param>
	/// <param name="e">The EventArgs</param>
	protected void Page_Load(object sender, EventArgs e)
	{
		try
		{
			if (!string.IsNullOrEmpty(Request.QueryString["ajaxAction"]))
			{
				Response.Clear();
				System.Web.Script.Serialization.JavaScriptSerializer json = new System.Web.Script.Serialization.JavaScriptSerializer();

				switch (Request.QueryString["ajaxAction"])
				{
					case "ContactSearch":
						//get the search term from the request
						string ContactSearch = Request.QueryString["term"];
						//use the LoadContactLookUpList function in the web service to get a sorted list of contacts that match the search term
						Contact[] Contacts = ws.LoadContactLookUpList(new[] { new Filter { FieldName = "name", Operand = "LIKE", SearchValue = ContactSearch } });
						//respond with a json serialized array with the contact id and label for each result
						Response.Write(json.Serialize((
							from c in Contacts
							select new
							{
								id = Convert.ToInt32(c.Fields.First(f => f.FieldName == "contact_id").FieldValue),
								label = Convert.ToString(c.Fields.First(f => f.FieldName == "name").FieldValue)
							}).ToArray()));
						break;
					case "ProjectSearch":
						//get the search term from the request
						string ProjectSearch = Request.QueryString["term"];
						//use the GetProjectListWithoutParentsOrChildren function to get a list of projects that match the search directly
						Task[] Tasks = ws.GetProjectsListWithoutParentsOrChildren(
							new[] { new Field { FieldName = "job_id" }, new Field { FieldName = "brief_description" } },
							new[] { new Filter { FieldName = "brief_description", Operand = "LIKE", SearchValue = ProjectSearch } }, 0,
							new[] { new OrderBy { FieldName = "brief_description", AscendingOrder = true } },
							false, false);
						//return a json serialized array with the project id and label for each result
						Response.Write(json.Serialize((
							from t in Tasks
							select new
							{
								id = Convert.ToInt32(t.Fields.First(f => f.FieldName == "job_id").FieldValue),
								label = Convert.ToString(t.Fields.First(f => f.FieldName == "brief_description").FieldValue)
							}).ToArray()));
						break;
				}
				Response.End();
			}
		}
		catch (Exception ex)
		{
			ShowError(ex);
		}
	}

	/// <summary>
	/// Builds the task type forms and field controls
	/// </summary>
	protected override void CreateChildControls()
	{
		try
		{
			//check for the login flag in the query string. If it is there, show just the login control
			if (Login.Visible = LoginMode)
				return;
			
			//if the query string contains the "taskId" parameter, then load the task to be edited
			if (int.TryParse(Request.QueryString["taskId"], out EditTaskId))
			{
				Task ThisTask = ws.OpenTask(EditTaskId, new Field[0], true);
				TaskTypeId = (ThisTask.Fields.First(f => f.FieldName == "job_type").FieldValue as TaskType).Id;
				TranslatedTaskValues = GetTranslatedTaskValues(ThisTask);
			}
			else
			{
				//we are in create task mode, load the task type from the query string
				EditTaskId = -1;
				if (!int.TryParse(Request.QueryString["taskType"], out TaskTypeId))
					throw new ApplicationException("The querystring must contain either a valid \"taskType\" id or a valid \"taskId\"");
			}

			//load the forms and fields for the task type
			TaskType Type = ws.GetTaskTypeInfo(TaskTypeId, JobTypeDisplayFor.TasksAndProjects);

			HtmlTable FormTable;
			int FormCount = 1;
			
			//loop through each form and load a separate tab for each. Each tab holds the fields for that form
			foreach (JTFormInfo Form in Type.Forms)
			{
				//add the current form tab and table
				HtmlGenericControl FormHeader = new HtmlGenericControl("li");
				FormHeader.Controls.Add(new HtmlAnchor
				{
					HRef = "#forms-" + FormCount,
					InnerText = Form.FormName
				});
				formHeaders.Controls.Add(FormHeader);
				FormTable = new HtmlTable();
				FormTable.Attributes["class"] = "formTable";
				HtmlGenericControl CurrForm = new HtmlGenericControl("div") { ID = "forms-" + FormCount++ };
				CurrForm.Controls.Add(FormTable);
				forms.Controls.Add(CurrForm);

				//loop through the fields in this form and build the control for each
				foreach (FormFieldInfo Field in Form.FormFields)
				{
					//the FormFieldInfo object does not have all the information (such as permissions and display name). To get those
					//load the TaskField object from the TaskField list
					TaskField FieldInfo = TypeFields.First(f => f.FieldID == Field.FieldID);
					
					//build the control. Skip this field if it is not supported or if the user has no edit permission.
					Control FieldControl = BuildJTControl(FieldInfo);
					if (FieldControl == null || !FieldInfo.CanEdit)
						continue;

					//build the html for the field row and label
					HtmlTableRow FieldRow = new HtmlTableRow();
					FormTable.Rows.Add(FieldRow);

					HtmlTableCell LabelCell = new HtmlTableCell { InnerText = FieldInfo.FieldDisplayName };
						
					if (Field.IsRequired)
						LabelCell.InnerHtml += " <span class='required'>*</span>";
					LabelCell.Attributes["class"] = "labelCell";
					FieldRow.Cells.Add(LabelCell);

					HtmlTableCell FieldCell = new HtmlTableCell();
					FieldCell.Attributes["class"] = "controlCell";
					FieldRow.Cells.Add(FieldCell);

					//handle specific field types and specific field options
					if (FieldInfo.DisplayFieldType == "Section Header")
					{
						LabelCell.Visible = false;
						FieldCell.ColSpan = 2;
					}
					else if (FieldInfo.DisplayFieldType == "Yes-No Radio Button List")
						(FieldControl as JTRadioButtonList).Items.AddRange(new[] { new ListItem("Yes", "1"), new ListItem("No", "0") });
					else if (FieldInfo.FieldName == "status")
						(FieldControl as JTDropDownList).Items.AddRange(Array.ConvertAll(ws.LoadStatuses(TaskTypeId), s => new ListItem(s.DisplayName, s.Id.ToString())));
					else if (FieldInfo.FieldName == "priority")
						(FieldControl as JTDropDownList).Items.AddRange(Array.ConvertAll(ws.LoadPriorities(), p => new ListItem(p.DisplayName, p.Id.ToString())).Reverse().ToArray());

					//if this is a JTControl, then add it to the JTControls collection for saving the values
					if (FieldControl is JTControl)
					{
						FieldControl.ID = Field.FieldName;
						JTControls.Add(FieldControl);
						//load the field's default value for initial loads
						if (!Page.IsPostBack)
						{
							(FieldControl as JTControl).JTValue =
								EditTaskId == -1 ? ws.GetTaskTypeDefaultFieldValue(FieldInfo.FieldID, TaskTypeId) : TranslatedTaskValues[FieldInfo.FieldName];

							if (FieldInfo.FieldName == "show_on_calendar" && (FieldControl as JTControl).JTValue == "")
								(FieldControl as JTControl).JTValue = "0";
						}
					}

					FieldCell.Controls.Add(FieldControl);
				}
			}

			forms.Visible = (EditTaskId == -1 ? CreateButton : SaveButton).Visible = ResetButton.Visible = true;
		}
		catch (Exception ex)
		{
			ShowError(ex);
		}
	}

	/// <summary>
	/// Runs when the create button is clicked. Attempts to create a task from the JTControl values
	/// </summary>
	/// <param name="sender">The event sender</param>
	/// <param name="e">The event args</param>
	void CreateButton_Click(object sender, EventArgs e)
	{
		try
		{
			EnsureChildControls();

			//This holds the FieldChange values to be saved
			FieldChange[] Fields = new FieldChange[JTControls.Count + 1];
			
			//The job_type needs to be set manually since there is no job type control on the page
			Fields[0] = new FieldChange
			{
				FieldName = "job_type",
				FieldValue = TaskTypeId.ToString(),
				IsAppendChange = false
			};

			//loop through each JTControl and create a FieldChange for each control
			for (int i = 0; i < JTControls.Count; i++)
			{
				Fields[i + 1] = new FieldChange
				{
					FieldName = JTControls[i].ID,
					FieldValue = (JTControls[i] as JTControl).JTValue,
					IsAppendChange = false
				};
			}

			//Attempt to create the task. If any fields are invalid, this will thrown an exception which will be handled by ShowError.
			int TaskId = ws.CreateTask(Fields, false);
			
			//If the task creation succeeded, create a success popup.
			var SuccessBox = new HtmlGenericControl("div") { InnerHtml = "Task <b>#" + TaskId + "</b> was created.", ID = "messageBox" };
			SuccessBox.Attributes["title"] = "Task Created";
			SuccessBox.Attributes["clickAction"] = "reset";
			form1.Controls.Add(SuccessBox);
		}
		catch (Exception ex)
		{
			ShowError(ex);
		}
	}

	/// <summary>
	/// Runs when the save button is clicked. Attempts to save the task from the JTControl values
	/// </summary>
	/// <param name="sender">The event sender</param>
	/// <param name="e">The event args</param>
	void SaveButton_Click(object sender, EventArgs e)
	{
		try
		{
			EnsureChildControls();

			//This holds the FieldChange values to be saved
			List<FieldChange> Fields = new List<FieldChange>();

			//loop through each JTControl and create a FieldChange for each control
			for (int i = 0; i < JTControls.Count; i++)
			{
				FieldChange NewChange = new FieldChange
				{
					FieldName = JTControls[i].ID,
					FieldValue = (JTControls[i] as JTControl).JTValue,
					IsAppendChange = false
				};
				if (NewChange.FieldValue != TranslatedTaskValues[NewChange.FieldName])
					Fields.Add(NewChange);
			}

			//Attempt to save the task. If any fields are invalid, this will thrown an exception which will be handled by ShowError.
			ws.EditTask(EditTaskId, Fields.ToArray(), false);

			//If the task save succeeded, create a success popup.
			var SuccessBox = new HtmlGenericControl("div") { InnerHtml = "Task <b>#" + EditTaskId + "</b> was saved.", ID = "messageBox" };
			SuccessBox.Attributes["title"] = "Task Saved";
			SuccessBox.Attributes["clickAction"] = "reset";
			form1.Controls.Add(SuccessBox);
		}
		catch (Exception ex)
		{
			ShowError(ex);
		}
	}

	/// <summary>
	/// Runs when the login button is clicked. Verify the login and reload the form
	/// </summary>
	/// <param name="sender">The event sender</param>
	/// <param name="e">The event args</param>
	private void Login_Authenticate(object sender, AuthenticateEventArgs e)
	{
		try
		{
			Session["Login"] = ws.UserLogin(Login.UserName, Login.Password);
			NameValueCollection NewQuery = HttpUtility.ParseQueryString(Request.Url.Query);
			NewQuery.Remove("login");
			Response.Redirect(Request.Path + "?" + NewQuery.ToString(), true);
		}
		catch (Exception ex)
		{
			ShowError(ex);
		}
	}

	/// <summary>
	/// Get's the string value for the fields in the supplied Task
	/// </summary>
	/// <param name="t">The Task to load fields from</param>
	/// <returns>A Dictionary holding the field names as the key and the string value as the value</returns>
	Dictionary<string, string> GetTranslatedTaskValues(Task t)
	{
		var Fields = new Dictionary<string, string>();
		foreach (Field f in t.Fields)
		{
			string Value = null;
			switch (f.FieldDataType)
			{
				case "int":
				case "string":
				case "dateTime":
				case "double":
					Value = Convert.ToString(f.FieldValue, System.Globalization.CultureInfo.InvariantCulture);
					break;
				case "ArrayOfContact":
					Contact[] Contacts = (f.FieldValue as Contact[]);
					if (Contacts != null && Contacts.Length > 0)
						Value = string.Join(";", Array.ConvertAll(Contacts, c => c.Fields.First(cf => cf.FieldName == "contact_id").FieldValue.ToString()));
					else
						Value = "-1";
					break;
				case "Contact":
					Value = (f.FieldValue as Contact).Fields.First(cf => cf.FieldName == "contact_id").FieldValue.ToString();
					break;
				case "Status":
					Value = (f.FieldValue as Status).Id.ToString();
					break;
				case "Priority":
					Value = (f.FieldValue as Priority).Id.ToString();
					break;
				case "boolean":
					Value = Convert.ToInt32(f.FieldValue).ToString();
					break;
			}
			if (Value != null)
				Fields[f.FieldName] = Value;
		}
		return Fields;
	}

	/// <summary>
	/// Builds a control from the specified TaskField info
	/// </summary>
	/// <param name="fieldInfo">The TaskField object holding info about the field</param>
	/// <returns>The appropriate Control for the field</returns>
	private Control BuildJTControl(TaskField fieldInfo)
	{
		switch (fieldInfo.DisplayFieldType)
		{
			case "Append Text Area":
			case "Text Area":
				return new JTTextBox { CssClass = "textArea", TextMode = TextBoxMode.MultiLine, Rows = 4 };
			case "Project Lookup Field":
				return new JTLookup(JTLookup.Type.Project,
					//this function loads the project name from the specified id
					id => id == -1 ? "" : ws.OpenProject(
						id,
						new[] { new Field { FieldName = "brief_description" } },
						false
					).Fields.First(f => f.FieldName == "brief_description").FieldValue.ToString(),
					false
				);
			case "Contact Lookup Field":
				return new JTLookup(JTLookup.Type.Contact,
					//this fucntion loads the contact name from the specified id
					id => id == -1 ? "" : ws.OpenContact(
						id,
						new[] { new Field { FieldName = "name" } },
						false
					).Fields.First(f => f.FieldName == "name").FieldValue.ToString(),
					fieldInfo.FieldName == "assignee" || fieldInfo.FieldName == "other_notification_recipient"
				);
			case "Date and Time Textbox":
				return new JTDateTimeTextBox();
			case "Date Textbox":
				return new JTDateTextBox();
			case "Dropdown Menu":
				JTDropDownList DropDown = new JTDropDownList { CssClass = "dropdown" };
				//add a blank field option for normal dropdowns
				if (fieldInfo.DatabaseType == "string")
					DropDown.Items.Add("");
				//load the ListItems from the Options property of the TaskField object
				DropDown.Items.AddRange(
					fieldInfo.Options.Where(o => !o.Deleted)
					.Select(o =>
					{
						var li = new ListItem(o.OptionName);
						li.Attributes["isHeader"] = o.OptionIsHeader.ToString();
						li.Attributes["header"] = o.Header;
						return li;
					}).ToArray());
				return DropDown;
			case "Textbox":
			case "Email Address Textbox":
			case "Numeric Textbox":
			case "URL Textbox":
				return new JTTextBox { CssClass = "textBox" };
			case "Horizontal Checkbox List":
			case "Vertical Checkbox List":
				CheckBoxList CheckBox = new JTCheckBoxList { RepeatLayout = RepeatLayout.Flow };
				//load the ListItems from the Options property of the TaskField object
				CheckBox.Items.AddRange((from o in fieldInfo.Options where !o.Deleted select new ListItem(o.OptionName)).ToArray());
				if (fieldInfo.DisplayFieldType == "Horizontal Checkbox List")
					CheckBox.RepeatDirection = RepeatDirection.Horizontal;
				return CheckBox;
			case "Yes-No Radio Button List":
			case "Horizontal Radio Button List":
			case "Vertical Radio Button List":
				JTRadioButtonList RadioButton = new JTRadioButtonList { RepeatLayout = RepeatLayout.Flow };
				//load the ListItems from the Options property of the TaskField object
				RadioButton.Items.AddRange((from o in fieldInfo.Options where !o.Deleted select new ListItem(o.OptionName)).ToArray());
				if (fieldInfo.DisplayFieldType != "Vertical Radio Button List")
					RadioButton.RepeatDirection = RepeatDirection.Horizontal;
				return RadioButton;
			case "Section Header":
				return new JTSectionHeader { HeaderText = fieldInfo.FieldDisplayName };
			case "Timespan Textbox":
				return new JTTimespanTextbox();
			case "Dynamic Link":
				DynamicLinkInfo[] DynamicLinks = ws.GetAllDynamicLinks(DynamicLinkFor.Tasks);
				var LinkInfo = DynamicLinks.First(f => f.FieldID == fieldInfo.FieldID);
				foreach (FieldPart p in ws.GetUrlValuesForDynamicLink(DynamicLinkFor.Tasks, EditTaskId, fieldInfo.FieldName).FieldDataColl)
					LinkInfo.LinkURL = LinkInfo.LinkURL.Replace(p.Field, p.Part);
				return new DynamicLink { LinkName = (LinkInfo.LinkName == "" ? fieldInfo.FieldDisplayName : LinkInfo.LinkName), LinkUrl = LinkInfo.LinkURL };
			case "Embedded Dynamic Page":
				EmbeddedDynamicPageInfo[] EmbeddedPages = ws.GetAllEmbeddedDynamicPages(EmbeddedDynamicPageFor.Tasks);
				var PageInfo = EmbeddedPages.First(f => f.FieldID == fieldInfo.FieldID);
				foreach (FieldPart p in ws.GetUrlValuesForEmbeddedDynamicPage(EmbeddedDynamicPageFor.Tasks, EditTaskId, fieldInfo.FieldName).FieldDataColl)
					PageInfo.PageURL = PageInfo.PageURL.Replace(p.Field, p.Part);
				return new DynamicLink { LinkName = (PageInfo.PageName == "" ? fieldInfo.FieldDisplayName : PageInfo.PageName), LinkUrl = PageInfo.PageURL };
			default:
				return null;
		}
	}

	/// <summary>
	/// Display a formatted error message including the extra data if it is a SoapException
	/// </summary>
	/// <param name="ex">The Exception to display</param>
	private void ShowError(Exception ex)
	{
		StringBuilder Message = new StringBuilder();

		//if this is a SoapException, load the details from the embedded xml
		if (ex is System.Web.Services.Protocols.SoapException)
		{
			var Detail = (ex as System.Web.Services.Protocols.SoapException).Detail;

			if (Detail.FirstChild != null)
			{
				foreach (System.Xml.XmlNode Error in Detail.FirstChild.ChildNodes)
				{
					foreach (System.Xml.XmlNode Data in Error.ChildNodes)
					{
						switch (Data.Name)
						{
							case "ErrorState":
								if (Message.Length == 0)
									Message.Append("<i>" + Server.HtmlEncode(Data.InnerText) + "</i><br><br>");
								break;
							case "field_name":
								try
								{
									Message.Append("<b>Field</b>: " +
										Server.HtmlEncode(TypeFields.First(f => f.FieldName == Data.InnerText).FieldDisplayName) +
										"<br>");
								}
								catch (Exception) { }
								break;
							case "error_message":
								Message.Append("<b>Message</b>: " + Server.HtmlEncode(Data.InnerText) + "<br>");
								break;
							case "ServerErrorInformation":
								Message.Append(Server.HtmlEncode(ex.Message).Replace("\n", "<br>"));
								break;
						}
					}
					Message.Append("<br>");
				}
			}
			else
				Message.Append(Server.HtmlEncode(ex.Message).Replace("\n", "<br>")); //show the error message for soap exceptions without details errors
		}
		else
			Message.Append(Server.HtmlEncode(ex.Message).Replace("\n", "<br>")); //show the error message for normal errors

		//create the error box to hold the error message
		var ErrorBox = new HtmlGenericControl("div") { InnerHtml = Message.ToString(), ID = "messageBox" };
		ErrorBox.Attributes["title"] = "Error Message";
		form1.Controls.Add(ErrorBox);
	}

	/// <summary>
	/// Defines the JTControl interface. Implement on web controls that can get and set a field value for JobTraQ.
	/// </summary>
	private interface JTControl
	{
		/// <summary>
		/// Gets/Sets a string representation of the current control's value translated for use with the web service
		/// </summary>
		string JTValue { get; set; }
	}

	/// <summary>
	/// A simple JTControl wrapper for the WebControls TextBox
	/// </summary>
	private class JTTextBox : TextBox, JTControl
	{
		/// <summary>
		/// The JTValue is the current control text
		/// </summary>
		public string JTValue
		{
			get { return this.Text == "" ? null : this.Text; }
			set { this.Text = value; }
		}
	}

	/// <summary>
	/// Adds JTControl support to the WebControls DropDownList. Also adds support for optgroups
	/// </summary>
	private class JTDropDownList : DropDownList, JTControl
	{
		/// <summary>
		/// The JTValue is the currently selected option value
		/// </summary>
		public string JTValue
		{
			get { return this.SelectedValue; }
			set { this.SelectedValue = value; }
		}

		/// <summary>
		/// Renders the optios for this drop down. Writes optgroup tags fore each ListItem with the "isHeader" attribute
		/// </summary>
		/// <param name="writer">The output HtmlTextWriter</param>
		protected override void RenderContents(HtmlTextWriter writer)
		{
			string CurrHeader = "";

			foreach (ListItem i in this.Items)
			{
				if (i.Attributes["header"] != CurrHeader)
				{
					writer.WriteEndTag("optgroup");
					CurrHeader = "";
				}
				if (i.Attributes["isHeader"] == "True")
				{
					writer.WriteBeginTag("optgroup");
					writer.WriteAttribute("label", i.Text, true);
					writer.Write(HtmlTextWriter.TagRightChar);
					CurrHeader = i.Text;
				}
				else
				{
					writer.WriteBeginTag("option");
					if (i.Selected)
						writer.WriteAttribute("selected", "selected");
					writer.WriteAttribute("value", i.Value, true);
					writer.Write(HtmlTextWriter.TagRightChar);
					HttpUtility.HtmlEncode(i.Text, writer);
					writer.WriteEndTag("option");
				}
			}

			if (CurrHeader != "")
				writer.WriteEndTag("optgroup");
		}
	}

	/// <summary>
	/// A simple JTControl wrapper for the WebControls RadioButtonList
	/// </summary>
	private class JTRadioButtonList : RadioButtonList, JTControl
	{
		/// <summary>
		/// The JTValue is the currently selectted list value
		/// </summary>
		public string JTValue
		{
			get { return this.SelectedValue; }
			set { this.SelectedValue = value; }
		}
	}

	/// <summary>
	/// A simple JTControl wrapper for the WebControls CheckBoxList
	/// </summary>
	private class JTCheckBoxList : CheckBoxList, JTControl
	{
		/// <summary>
		/// The JTValue is a joined list of the currently seleted option values delimited by a comma [,]
		/// </summary>
		public string JTValue
		{
			get
			{
				return string.Join(",",
					(from ListItem i in this.Items
					 where i.Selected
					 select i.Value).ToArray());
			}
			set
			{
				foreach (string i in value.Split(','))
				{
					ListItem Item = this.Items.FindByValue(i);
					if (Item != null)
						Item.Selected = true;
				}
			}
		}
	}

	/// <summary>
	/// Creates a section header. Does not implement JTControl because it has no value
	/// </summary>
	private class JTSectionHeader : HtmlGenericControl
	{
		/// <summary>
		/// Gets/Sets the header text for the section header
		/// </summary>
		public string HeaderText { get; set; }

		/// <summary>
		/// Renders the HTML for this section header
		/// </summary>
		/// <param name="writer">The output HtmlTextWriter</param>
		protected override void Render(HtmlTextWriter writer)
		{
			writer.Write("<div class='sectionHeader'><hr /><div><span>");
			writer.WriteEncodedText(this.HeaderText);
			writer.Write("</span></div></div>");
		}
	}

	/// <summary>
	/// A special TextBox with jQuery date picker support
	/// </summary>
	private class JTDateTextBox : TextBox, JTControl
	{
		/// <summary>
		/// Create a new date text box instance
		/// </summary>
		public JTDateTextBox()
		{
			//The css class is used by jQuery to set up the date picker
			this.CssClass = "dateTextBox";
		}

		/// <summary>
		/// Attempts to get/set the short date value. If the value is not a valid DateTime, gets/sets an empty string
		/// </summary>
		public string JTValue
		{
			get
			{
				DateTime DateValue;
				if (DateTime.TryParse(this.Text, out DateValue))
					return DateValue.ToShortDateString();
				return "";
			}
			set
			{
				DateTime DateValue;
				if (DateTime.TryParse(value, out DateValue))
					this.Text = DateValue.ToShortDateString();
				else
					this.Text = "";
			}
		}
	}

	/// <summary>
	/// A JTControl that allows you to set a specific date and time
	/// </summary>
	private class JTDateTimeTextBox : CompositeControl, JTControl
	{
		JTDateTextBox DateBox = new JTDateTextBox();
		TextBox TimeBox = new TextBox { CssClass = "timeTextBox" };

		/// <summary>
		/// Adds the date and time textboxes
		/// </summary>
		protected override void CreateChildControls()
		{
			this.Controls.Add(DateBox);
			this.Controls.Add(new LiteralControl(" "));
			this.Controls.Add(TimeBox);
		}

		/// <summary>
		/// Attempts to get/set the date value. If the value is not a valid DateTime, gets/sets an empty string
		/// </summary>
		public string JTValue
		{
			get
			{
				DateTime DateValue;
				if (DateTime.TryParse(DateBox.Text + " " + TimeBox.Text, out DateValue))
					return TimeZone.CurrentTimeZone.ToUniversalTime(DateValue).ToString();
				return "";
			}
			set
			{
				DateTime DateValue;
				if (DateTime.TryParse(value, out DateValue))
				{
					DateValue = TimeZone.CurrentTimeZone.ToLocalTime(DateValue);
					DateBox.Text = DateValue.ToShortDateString();
					TimeBox.Text = DateValue.ToShortTimeString();
				}
				else
					DateBox.Text = TimeBox.Text = "";
			}
		}
	}

	/// <summary>
	/// A lookup textbox that uses AJAX to find a specific value
	/// </summary>
	private class JTLookup : CompositeControl, JTControl
	{
		Panel LookupBox = new Panel();
		HiddenField ValueBox = new HiddenField { ID = "ValueBox" };

		Func<int, string> GetLabelFromId;

		/// <summary>
		/// The available JTLookup types
		/// </summary>
		public enum Type
		{
			Project,
			Contact
		}

		/// <summary>
		/// Creates a new JTLookup control.
		/// </summary>
		/// <param name="type">The type of JTLookup to create</param>
		/// <param name="getLabelFromId">A function that loads the matching label from a specified ID</param>
		public JTLookup(Type type, Func<int, string> getLabelFromId, bool isMultiple)
		{
			LookupBox.Attributes["lookupType"] = type.ToString();
			LookupBox.Attributes["isMultiple"] = isMultiple.ToString().ToLower();
			this.GetLabelFromId = getLabelFromId;
		}

		/// <summary>
		/// Adds the child controls and sets the necessary parameters
		/// </summary>
		protected override void CreateChildControls()
		{
			this.Controls.Add(LookupBox);
			this.Controls.Add(ValueBox);

			//jQuery sets up the lookup using this css class
			this.LookupBox.CssClass = "jtLookup";

			//Associates the the lookukp box with the value box for client use
			LookupBox.Attributes["valueBox"] = ValueBox.ClientID;
		}

		/// <summary>
		/// Builds the initial items in the lookup based on the current ValueBox Value
		/// </summary>
		/// <param name="e">The EventArgs</param>
		protected override void OnPreRender(EventArgs e)
		{
			int Id;
			foreach (string sid in ValueBox.Value.Split(';'))
			{
				if (int.TryParse(sid, out Id) && Id != -1)
				{
					HtmlGenericControl Item = new HtmlGenericControl("a") { InnerText = this.GetLabelFromId(Id) };
					Item.Attributes["recordId"] = Id.ToString();
					Item.Attributes["tabIndex"] = "-1";
					Item.Attributes["href"] = "javascript:void(0)";
					LookupBox.Controls.Add(Item);
				}
			}
		}

		/// <summary>
		/// Gets/Sets the selected id of the lookup
		/// </summary>
		public string JTValue
		{
			get { return ValueBox.Value == "" ? "-1" : ValueBox.Value; }
			set { ValueBox.Value = value; }
		}
	}

	/// <summary>
	/// A JTControl for setting  a timespan of hours and minutes
	/// </summary>
	private class JTTimespanTextbox : CompositeControl, JTControl
	{
		TextBox HourBox = new TextBox { ID = "HourBox", CssClass = "hourBox" };
		TextBox MinuteBox = new TextBox { ID = "MinuteBox", CssClass = "minuteBox" };

		/// <summary>
		/// Add the hour and minute boxes
		/// </summary>
		protected override void CreateChildControls()
		{
			this.Controls.Add(HourBox);
			this.Controls.Add(new Label { Text = " hrs ", AssociatedControlID = "HourBox" });

			this.Controls.Add(MinuteBox);
			this.Controls.Add(new Label { Text = " min", AssociatedControlID = "MinuteBox" });
		}

		/// <summary>
		/// Gets/Sets a string value holding the total minutes for the timespan
		/// </summary>
		public string JTValue
		{
			get
			{
				int Hours = 0, Minutes = 0;
				bool HourValid = int.TryParse(HourBox.Text, out Hours);
				bool MinValid = int.TryParse(MinuteBox.Text, out Minutes);
				if (!HourValid && !MinValid)
					return "";
				return (Hours * 60 + Minutes).ToString();
			}
			set
			{
				try
				{
					int TotalMin = Convert.ToInt32(value);
					HourBox.Text = (TotalMin / 60).ToString();
					MinuteBox.Text = (TotalMin % 60).ToString();
				}
				catch
				{
					HourBox.Text = MinuteBox.Text = "";
				}
			}
		}
	}

	/// <summary>
	/// Renders the dynamic link control.
	/// </summary>
	private class DynamicLink : CompositeControl
	{
		HtmlAnchor link = new HtmlAnchor();

		/// <summary>
		/// 
		/// </summary>
		public DynamicLink()
		{
		}

		/// <summary>
		/// Adds the link and its url.
		/// </summary>
		protected override void CreateChildControls()
		{
			HtmlGenericControl linkContainer = new HtmlGenericControl("div");
			
			link.Target = "_blank";
			link.HRef = this.LinkUrl;
			link.InnerText = this.LinkName;
			linkContainer.Attributes.Add("class", "dynamicLink");

			linkContainer.Controls.Add(link);
			this.Controls.Add(linkContainer);
		}

		/// <summary>
		/// Gets/Sets a string value holding link url for the dynamic link or embedded page field.
		/// </summary>
		public string LinkUrl { get; set; }

		/// <summary>
		/// Gets/Sets a string value holding link display name string for the dynamic link or embedded page field.
		/// </summary>
		public string LinkName { get; set; }
		
	}

</script>

<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml">
<head runat="server">
	<title>JobTraQ Web Service Example Form</title>
	<link href="http://ajax.googleapis.com/ajax/libs/jqueryui/1.8.7/themes/redmond/jquery-ui.css" rel="Stylesheet" />
	<link href="StyleSheet.css" rel="Stylesheet" />

	<script type="text/javascript" src="https://ajax.googleapis.com/ajax/libs/jquery/1.4.4/jquery.min.js"></script>
	<script type="text/javascript" src="https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.7/jquery-ui.min.js"></script>
	<script type="text/javascript" src="ClientScript.js"></script>

</head>
<body>
	<form id="form1" runat="server">
	<div id="outer">
		<div id="header">
			<img src="Logo.gif" />Web Service Example Form
		</div>
		<asp:Login ID="Login" runat="server" DisplayRememberMe="false" OnAuthenticate="Login_Authenticate" Visible="false" TitleText="" />
		<div id="forms" runat="server" Visible="false">
			<ul id="formHeaders" runat="server"></ul>
		</div>
		<br />
		<asp:Button runat="server" ID="CreateButton" Text="Create New Task" OnClick="CreateButton_Click" Visible="false" />
		<asp:Button runat="server" ID="SaveButton" Text="Save Task" OnClick="SaveButton_Click" Visible="false" />
		<asp:Button runat="server" ID="ResetButton" Text="Reset" OnClientClick="window.location = window.location.href; return false;" Visible="false" />
	</div>
	</form>
</body>
</html>
